#!/usr/bin/python3
# -*- coding: utf-8 -*-

# edcoRobot.py

from smbus import SMBus
import time,os 
import RPi.GPIO as GPIO

class EdcoRobot:
	def __init__(self):
		self.BUTTON_START = 17
		self.ir = SMBus(1)
		GPIO.setwarnings(False)
		GPIO.setmode(GPIO.BCM)
		GPIO.setup(self.BUTTON_START,GPIO.IN,pull_up_down=GPIO.PUD_UP)
		self.TRIG = 5                                   #Associate pin 5 to TRIG
		self.ECHO = 6                                   #Associate pin 6 to ECHO
		GPIO.setup(self.TRIG,GPIO.OUT)                  #Set pin as GPIO out
		GPIO.setup(self.ECHO,GPIO.IN)                   #Set pin as GPIO in
		GPIO.output(self.TRIG, False)                   #Set TRIG as LOW		

	def byIR(self):
		return self.ir.read_byte(0x39)

	def bySensors(self):
		byIr = self.byIR()
		
		bBR = byIr & 0x01
		bBL = (byIr >> 1) & 0x01
		bFSL = (byIr >> 2) & 0x01
		bFL = (byIr >> 3) & 0x01
		bFC = (byIr >> 4) & 0x01
		bFR = (byIr >> 5) & 0x01
		bFSR = (byIr >> 6) & 0x01
		
		return [bBR,bBL,bFSL,bFL,bFC,bFR,bFSR]

	def digitalRead(self,nPin):
		return GPIO.input(nPin)

	def vShowIR(self):
		bBR,bBL,bFSL,bFL,bFC,bFR,bFSR = self.bySensors()
		
		self.gotoxy(1,1);
		print("  { d: %d cm }            "%self.nUltrasonicCm() )
		print(" /----/^\\----\\");
		print(" |%c  %c %c %c  %c|"%('W' if bFSL else 'B','W' if bFL else 'B','W' if bFC else 'B','W' if bFR else 'B','W' if bFSR else 'B'))
		print("[ ]    ^    [ ]");
		print("[ ]    %c    [ ]"%('N' if self.digitalRead(self.BUTTON_START) else'P'))
		print(" |     ^     |");
		print("[ ]    ^    [ ]");
		print("[ ]    ^    [ ]");
		print(" |%c    ^    %c|"%('W' if bBL else'B','W' if bBR else 'B'))
		print(" +-----------+");


	def gotoxy(self,x,y):
		print ("%c[%d;%df" % (0x1B, y, x), end='') 

	def nUltrasonicCm(self):
		global TRIG,ECHO
		
		GPIO.output(self.TRIG, True)                  #Set TRIG as HIGH
		time.sleep(0.00001)                      #Delay of 0.00001 seconds
		GPIO.output(self.TRIG, False)                 #Set TRIG as LOW

		while GPIO.input(self.ECHO)==0:               #Check whether the ECHO is LOW
			pulse_start = time.time()              #Saves the last known time of LOW pulse

		while GPIO.input(self.ECHO)==1:               #Check whether the ECHO is HIGH
			pulse_end = time.time()                #Saves the last known time of HIGH pulse 

		pulse_duration = pulse_end - pulse_start #Get pulse duration to a variable

		distance = pulse_duration * 17150        #Multiply pulse duration by 17150 to get distance
		distance = round(distance, 2)            #Round to two decimal points

		return distance

if __name__ == "__main__":
	def main():
		os.system("clear")
		robot = EdcoRobot()
		try:
			while True:
				robot.vShowIR()
				time.sleep(0.1) 
				
		except KeyboardInterrupt:
			os.system("clear")
			print ("\nSortint ...")

	main()
